
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
# this script is assumed to be executed on Unix/Linux machine only to choose the apropriate gnu xdp to generate 'cgnu.xdp'

use File::Basename;
use File::Copy;

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
require $SCRIPTDIR . "Translate.pl";
require $SCRIPTDIR . "rsfGetInput.pl";

$ENV{LC_MESSAGES}='C'; #to get the gcc message without any localization
$ENV{LC_ALL}='C'; #to get the gcc message without any localization
my $INTDIR=".";
my $envPathSeparator=":";
   $envPathSeparator=";" if($^O=~/Win32/);
my $xdpFile="cgnu.xdp"; #xdp to generate
my $tmplog="$INTDIR/tmp_gcc_v.log";

my $gcc_dir;
my $gcc_version;
my $silent=0;

sub quit
{
	print &PerlTranslate("ConfigureGcc.pl", "\nConfiguration is not complete.\n");
	print &PerlTranslate("ConfigureGcc.pl", "You can relaunch this configuration script later by using the command:\n");
	print "$BINDIR/TestRTperl -I$ENV{'TESTRTDIR'}/lib/perl ConfigureGcc.pl\nor\nConfigureGcc.sh\n"; # NO TRANSLATION
	print &PerlTranslate("ConfigureGcc.pl", "Confirm quit ? ( y | n )\n");
	$reply='y';
	#$reply=<STDIN>;
	$reply=&rsfGetInput('gcc_configure_confirm_quit','') unless($silent);#no default- use for silent install
	chomp($reply);
	if(($reply eq &PerlTranslate("ConfigureGcc.pl", "y"))||($reply eq &PerlTranslate("ConfigureGcc.pl", "yes")))
	{
		print &PerlTranslate("ConfigureGcc.pl", "Quitting\n");
		exit(1);
	}
}

sub showLog #($)
{
	my $log=shift;

	open(IN,"<$log");
	while(<IN>) { print ; }
	close(IN);
}

sub extractVersion
{
my $log=shift;#logFile name
my $ret=0;
open(GCC,"<$log");
while(<GCC>)
{
	if (/^(.*) version (?:egcs-)?((\d+\.)+\d+).*$/)
	{
		print &PerlTranslate("ConfigureGcc.pl", "the path to gcc is %1\n", "%1 is a path", $p);
		if(/for\sGNAT/){
			print &PerlTranslate("ConfigureGcc.pl", "This gcc is for GNAT and cannot be used (ignored)\n");
		}
		else  {
			$ret=$2;
		}
		last;
	}
}
close(GCC);
return $ret;
}
sub enterPath
{
	print &PerlTranslate("ConfigureGcc.pl", "\nEnter the complete name of the directory containing gcc or q to quit\n");

	#$gcc_dir=<STDIN>;
	$gcc_dir=&rsfGetInput('gcc_complete_path','') unless($silent);#no default - use for silent install

	chomp($gcc_dir);
	if($gcc_dir eq &PerlTranslate("ConfigureGcc.pl", "q"))
	{
		quit();
		$gcc_dir="";
	}
}

sub CompilError
{
my $critical=shift;
print &PerlTranslate("ConfigureGcc.pl", "The return status was %1\n", "%1 is the status", $Status);
print &PerlTranslate("ConfigureGcc.pl", "%1 produced the following results\n", "%1 is the command that has been launched", "gcc -v $INTDIR/empty.c");
showLog($tmplog);
print &PerlTranslate("ConfigureGcc.pl", "\nConfiguration is not complete.\n");
print &PerlTranslate("ConfigureGcc.pl", "Please solve the problem with the gnu compiler. Contact your system administrator if necessary.\n");
print &PerlTranslate("ConfigureGcc.pl", "You can relaunch this configuration script later by using the command:\n");
print "$BINDIR/perllauncher ConfigureGcc.pl\nor\nConfigureGcc.sh\n"; # NO TRANSLATION
exit(1) if($critical);
}

sub MCheckGcc
{
	my $Status;
	my $gcccmd='';
	my $try=5;

	while($try)
	{
		$try--;
		$gcc_dir.='/' if($gcc_dir=~m/\w$/);
		$gcc_dir='/usr/bin/' if($gcc_dir eq '' && $gcccmd ne '');
		$gcccmd=$gcc_dir."gcc";
		my $p=`which $gcccmd` ;
		if($p !~m/gnat/i)
		{
			print &PerlTranslate("ConfigureGcc.pl", "Trying to execute %1\n", "%1 is the command to be executed", "$gcccmd -v");
			$back_lang = $ENV{'LANG'};
			$ENV{'LANG'} = "C";
			$Status=system("$gcccmd -v > $tmplog 2>&1");
			$ENV{'LANG'} = $back_lang;
			$Status>>=8 if($^O=~/Win32/);
			$gcc_version=0;
			unless($status)
			{
				$gcc_version=extractVersion($tmplog);
				if($gcc_version)
				{
					print &PerlTranslate("ConfigureGcc.pl", "Obtained following result:\n");
					showLog($tmplog);
					print &PerlTranslate("ConfigureGcc.pl", "\nIs this correct ? ( y | n )\n");
					$reply='y';
					#$reply=<STDIN>;
					$reply=&rsfGetInput('gcc_confirm_path','') unless($silent);#no default - use for silent install

					chomp($reply);
					if(($reply eq &PerlTranslate("ConfigureGcc.pl", "y"))||($reply eq &PerlTranslate("ConfigureGcc.pl", "yes")))
					{
						$try=0;
					}
					else
					{
						enterPath();
					}
				}
				else
				{
					print &PerlTranslate("ConfigureGcc.pl", "The return status was %1\n", "%1 is the status", $Status);
					print &PerlTranslate("ConfigureGcc.pl", "The command %1 does not work properly: Here is the log file:\n", "%1 is the command", $gcccmd);
					showLog($tmplog);
					enterPath();
				}
			}
		}
	}
	if($gcc_version)
	{
		print &PerlTranslate("ConfigureGcc.pl", "Trying to C compile\n");
		$ENV{'PATH'}=$gcc_dir.$envPathSeparator.$ENV{'PATH'} if($gcc_dir ne "");
		open(OU,">$INTDIR/empty.c"); print OU "int main() { return(0); }\n"; close(OU);
		$Status=system("gcc -v $INTDIR/empty.c > $tmplog 2>&1");
		CompilError(1) if($Status != 0);

		copy("$INTDIR/empty.c","$INTDIR/empty.cpp");
		print &PerlTranslate("ConfigureGcc.pl", "Trying to C++ compile\n");
		$Status=system("g++ -v $INTDIR/empty.cpp > $tmplog 2>&1");
		if($Status){
			CompilError(0);
		} else {
			unlink($tmplog);
			print &PerlTranslate("ConfigureGcc.pl", "Success\n") if($Status == 0);	
		}
		unlink("$INTDIR/empty.c");
		unlink("$INTDIR/empty.cpp");
		unlink("$INTDIR/a.out");
		return 1;
	}
	return 0;
}

#This patch was used to get xdp for 64 bits based on 32bits xdp file
#It is not any more used since Solaris,Aix and HP are not any more supported
#thsi code is kept in case thos support comes back 
sub patchXdp #($$$)	patchXdp("$xmlDir/$orgxdp","$xmlDir/$xdpFile",$gcc_dir,$patch64);
{
	my $orgxdp=shift;
	my $newxdp=shift;
	my $path=shift;
	my $patch64=shift;

	my $pathmet;
	my $beforeComment=1;
	my $inOpp=0;
	my $insizet=0;
	print &PerlTranslate("ConfigureGcc.pl", "Patching %1\n", "%1 is the file to be patched", $newxdp);
	print &PerlTranslate("ConfigureGcc.pl", "Patching with gcc PATH as %1\n", "%1 is the value for the gcc PATH", $path) if($path);
	print &PerlTranslate("ConfigureGcc.pl", "Patching %1 for 64bits machine\n", "%1 is the file to be patched",$newxdp) if($patch64);
	$pathmet=0;
	open(IN,"<$orgxdp");
	open(OU,">$newxdp");
	while(<IN>)
	{
		if(/^<TDP\s/) #change the TDP name to 'GNU'
		{
#			s/(Name=\"[^\"]*Gnu) [^\"]*\"/$1\"/;
			s/Name=\"[^\"]*\"/Name=\"GNU\"/;
		}
		elsif($beforeComment && /<COMMENT>/)
		{
			print OU "<COMMENT>xdp automatically generated based on $orgxdp\n";
			print OU "include modification for 64bits platform\n" if($patch64);
			s/<COMMENT>(.*)$/$1/;
			$beforeComment=0;
		}
		#modify the PATH to access gcc
		elsif(/<KEY>ENV_PATH<\/KEY>/)
		{
			$pathmet=1 if($path ne "");
		}
		elsif($pathmet==1)
		{
			s/<VALUE>.*<\/VALUE>/<VALUE>$path:\$ENV{'PATH'}<\/VALUE>/;
			$pathmet=0;
		}
		elsif($patch64) {
			#changes for 64bits machine
			if(/RTRT_64_BITS_PTR/) {
				s/Value=\"0\"/Value=\"1\"/;
			}
			elsif(/VALUE>\(\(RTRT_U_INT32\)p\)<\/VALUE/){
				s,VALUE>\(\(RTRT_U_INT32\)p\)<\/VALUE,VALUE>((RTRT_U_INT64)p)<\/VALUE,;
			}
			#change the RTRT_SIZE_T type
			elsif(/SIZET/) {
				$insizet=1;
			}
			elsif($insizet && /<VALUE>/) {
				$insizet=0;
				s/<VALUE>unsigned\s+int</<VALUE>unsigned long</;
			}
			#change options in atl.opp
			elsif(/<PCOPP /) {
				$inOpp=1;
			}
			elsif($inOpp && /(targ_size_t_int_kind\s+(\d+))/) {
				print OU "--diag_suppress 61,69\n";
				s/$1/targ_size_t_int_kind 8/;
				$inOpp=0;
			}
			elsif($inOpp && /<\/VALUE>/) {
				$inOpp=0;
				print OU "--diag_suppress 61,69\n";
				print OU "--targ_size_t_int_kind 8\n";
			}
		}

		print OU;
	}
	close(IN);
	close(OU);
}

sub main
{
	my $perl;
	my $outTargetsDir="$ENV{'TESTRTDIR'}/targets";
	my $xmlDir="$outTargetsDir/xml";
	my $outXmlDir=$xmlDir;
	my $orgxdp;

	my $machine=`uname -m`;

	die &PerlTranslate("ConfigureGcc.pl", "Environment variable TESTRTDIR not properly set. Cannot find targets directory. Exiting.\n") if(! -d "$ENV{'TESTRTDIR'}/targets");
	foreach my $arg (@ARGV) {
		if($arg=~/silent$/) {
			$silent=1;
		}elsif(-d $arg){
			$outTargetsDir=$arg;
			$outXmlDir="$outTargetsDir/xml";
			mkdir( $outXmlDir, 0777) unless(-d $outXmlDir);
		}
	}

	if($^O =~ /linux/i)
	{
	    if(-d "$ENV{'TESTRTDIR'}/bin/s390x")
	    {
		$host="linux";
		$perl="$ENV{'TESTRTDIR'}/bin/s390x/$host/TestRTperl";
		$orgxdp="czlinuxgnu.xdp";
	    }
	    else
	    {	#intel redhat & suse
		my $host;

		print &PerlTranslate("ConfigureGcc.pl", "\nLinux configuration for C / C++\n");
		$host="linux";
		$host .='_64' if($machine=~m/64/);
		$perl="$ENV{'TESTRTDIR'}/bin/intel/$host/TestRTperl";

		my $s=MCheckGcc();
		# Gcc exists and is located
		my ($major_version,$minor_version,$release_number)=split(/\./,$gcc_version);
		print &PerlTranslate("ConfigureGcc.pl", "Status of CheckGcc %1: major=%2 minor=%3 location=%4\n", "%1 is the status, %2 is the Gnu compiler major part of the version, %3 is its minor part of the version and %4 is its location", $s, $major_version, $minor_version, ($gcc_dir?$gcc_dir:"in PATH"));
#        $orgxdp="clinuxgnu3_2.xdp" if($major_version le "4");
#		print &PerlTranslate("ConfigureGcc.pl", "No TDP match the version of gcc: %1 - Available versions are 3.2, 3.3, 3.4 and 4.x\n", "%1 is the Gnu compiler version", $gcc_version) if($orgxdp eq "");
        $orgxdp="clinuxgnu3_2.xdp" if($orgxdp eq ''); #default xdp
		
		copy("$xmlDir/$orgxdp","$outXmlDir/clinuxgnu.xdp");
	    }
	}
	else
	{
	    # unknown machine but it may be a Unix like
		my $s=MCheckGcc();
		# Gcc exists and is located
		my ($major_version,$minor_version,$release_number)=split(/\./,$gcc_version);
		print &PerlTranslate("ConfigureGcc.pl", "Status of CheckGcc %1: major=%2 minor=%3 location=%4\n", "%1 is the status, %2 is the Gnu compiler major part of the version, %3 is its minor part of the version and %4 is its location", $s, $major_version, $minor_version, ($gcc_dir?$gcc_dir:"in PATH"));
        $orgxdp="clinuxgnu3_2.xdp" if($orgxdp eq ''); #default xdp
		copy("$xmlDir/$orgxdp","$outXmlDir/clinuxgnu.xdp");
		copy("$xmlDir/clinuxgnu64.xdp","$outXmlDir/clinuxgnu64.xdp") if($outXmlDir ne $xmlDir);
	}
	die &PerlTranslate("ConfigureGcc.pl", "No TDP match the version of gcc: %1\n", "%1 is the Gnu compiler version", $gcc_version) if(($orgxdp eq "") || !(-f "$xmlDir/$orgxdp"));
	
	my $patch64=0;
	if($machine=~m/64/){
		my $xdp64=$orgxdp;$xdp64=~s/\.xdp$/64.xdp/;
		if(-f "$xmlDir/$xdp64"){
			$orgxdp=$xdp64;
		} else {
			$xdp64=~s/64\./_64./;
			if(-f "$xmlDir/$xdp64"){
				$orgxdp=$xdp64 ;
			} else {
				$patch64=1;
			}
		}
	}
	patchXdp("$xmlDir/$orgxdp","$outXmlDir/$xdpFile",$gcc_dir,$patch64);
	# generate the default cgnu TDP
	system("$perl -I$ENV{'TESTRTDIR'}/lib/perl $ENV{'TESTRTDIR'}/lib/scripts/tdpgen.pl $outXmlDir/$xdpFile  $outTargetsDir");
}

$|=1;
main();
1;
