#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2008-2016 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#            -*-  Source Instrumentation launcher for Rhapsody -*-             #
#                                                                              #
#                                                                              #
################################################################################

# generate the local TestRT.bat to set compiler/linker and ATLTGT
# then generate the option file
# then check that %1.bat has enough %x on the make command line
#RhapTestRT.exe -bat "%optionFile%.opt" "%RHAPSODYSHAREETC%\Env2TDP.txt" makefile.mak

die "Environment variable TESTRTDIR not found. Exiting.\n\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";
require $SCRIPTDIR."Fcts.pl";
################################################################################
# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle;
use File::Copy ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

$RHAP_NAME="root";
#$RHAP_FILE='HomeAlarm.rpy';
#$RHAP_DIR='.';
$RHAP_CONF_PATH=".";
$RHAP_LANG='CPP';
$RHAP_MAKEFILE='';
$RHAP_ENVIRONMENT="";
$RHAP_COV='-BLOCK=Logical -Proc=Ret';
$RHAP_MEMPRO='False';
$RHAP_PERFPRO='False';
$RHAP_TRACE='False';
$RHAP_LIBS='$OMROOT';
$RHAP_USEROPTIONS='';
$RHAP_USERSETTINGS='';
$RHAP_TARGET='<default>';
%RHAP_TAGS=();
$RHAPCONF_FAILED=0;
my $pwd=GetCurrentDir();
print "$pwd/RhapConf.pl generated by RhapConf.exe does not exist\n" unless(-f "RhapConf.pl");
require "RhapConf.pl";


my $batType=$ARGV[0];
my $optionFile=$ARGV[1]; $optionFile=~s/\"//g;
my $eqFile=$ARGV[2];     $eqFile=~s/\"//g;
my $makFile=$ARGV[3]; $makFile=~s/\"//g;

#print "+$RHAP_ENVIRONMENT\n";
#print "+$RHAP_CONF_PATH\n";
#print "+$RHAP_NAME\n";
#print "+$optionFile\n";
#print "+$eqFile\n";
#print "+$makFile\n";

#create directory for TestRT static files
my $TestRTstaticDir=$RHAP_CONF_PATH.'/TestRT';
unless(-d $TestRTstaticDir)  {
	mkdir ($TestRTstaticDir,0777);
}

# load defaults TDP list according to Rhapsody property CPP_CG.Configuration.environment
my $CC="";#="\$(CC)";
my $CPP="";#="\$(CPP)";
my $LINK_CMD="";#="\$(LINK_CMD)";
my $LINK_CMD_VAR="LINK_CMD";#may be changed by eqFile
my $CPP_VAR="CPP";#may be changed by eqFile
my $CC_VAR="CC";#may be changed by eqFile
my $BATCH_CMD="";#name of the batch used to build which has been replaced by TestRT.bat;
my $firstEqLine;

unless($RHAP_ENVIRONMENT){ # Env not found. try to recover using other settings
	$RHAP_ENVIRONMENT='Cygwin' if($makFile=~m/[\\\/]cygwin/i);
	$RHAP_ENVIRONMENT='MSVC' if($makFile=~m/[\\\/]msvc/i);
	$RHAP_ENVIRONMENT='QNX' if($makFile=~m/[\\\/]qnx/i);
	$RHAP_ENVIRONMENT='WorkbenchManaged' if($makFile=~m/[\\\/]vx/i);
}

if(open (TDP, "$eqFile")){
  while(<TDP>){
    next if(/^#/);
    $firstEqLine=$_ unless($firstEqLine);
    if(/^\s*\Q$RHAP_ENVIRONMENT\E\s+(.*)$/){
    	my $values=$1.' ';
    	print "Environment found as $values\n";
    	readEqLine($values);
    	last;
    }
  }
  close TDP;
} else { print "unable to read $eqFile. Check installation.\n";}

unless($ENV{'ATLTGT'}){ # TDP not found. try to use the first TDP in the EQ list
	print "Unable to recover TDP from Rhapsody environment!!\n  Use first TDP of $eqFile\n";
	readEqLine($firstEqLine.' ');
}

print "use instrumentation options from $pwd/$optionFile\n" if($RHAPCONF_FAILED);
unless($RHAPCONF_FAILED && (-f $optionFile)) {
#write option file regarding RTRT_xx tags and properties
  if(open (OPT,">$optionFile")) {
     print OPT "--NAME=$RHAP_NAME\n";
     print OPT "-FDCDIR=\"$TestRTstaticDir\"\n-TSFDIR=\"$TestRTstaticDir\"\n";

     foreach my $tag (keys %RHAP_TAGS){
	if($tag=~/^RTRT_/){
		if($tag eq "RTRT_Target"){
			print "Target forced by tag RTRT_Target\n";
			my $tgt=$RHAP_TAGS{$tag};
			unless($tgt=~m/[\\\/]/){
				my $xdpName=$tgt;
				$tgt=getAppData().'/targets/'.$tgt;
				$tgt=$TESTRTDIR  .'/targets/'.$xdpName unless(-e $tgt."/tp.ini");
			}
			if(-d $tgt) {
			    $ENV{'ATLTGT'}=$tgt;
			    $CC=""; $CPP=""; $LINK_CMD="";
			    print "setting env ATLTGT=$tgt\n";
			} else {
			    print "Unnown TDP ",$tgt,"\n";
			}
		}
		elsif($tag eq "RTRT_CC"){
			$CC=$RHAP_TAGS{$tag};
		}
		elsif($tag eq "RTRT_CPP"){
			$CPP=$RHAP_TAGS{$tag};
		}
		elsif($tag eq "RTRT_LINK_CMD"){
			$LINK_CMD=$RHAP_TAGS{$tag};
		}
		else
		{
			print OPT $RHAP_TAGS{$tag},"\n";
		}
	}
     }
     if(($RHAP_TARGET ne "<default>") && ($RHAP_TARGET ne "")){
	print "Target forced by Target property\n";
	my $tgt=$RHAP_TARGET;
	unless($tgt=~m/[\\\/]/){
		my $xdpName=$tgt;
		$tgt=getAppData().'/targets/'.$tgt;
		$tgt=$TESTRTDIR  .'/targets/'.$xdpName unless(-e $tgt."/tp.ini");
	}
	if(-d $tgt) {
	    $ENV{'ATLTGT'}=$tgt;
	    $CC=""; $CPP=""; $LINK_CMD="";
	    print "setting env ATLTGT=$tgt\n";
	} else {
	    print "Unnown TDP ",$tgt,"\n";
	}
     }

     if($batType=~m/f$/i){ #batF or shF for the frameWork build
	print OPT "-memprolib " if($RHAP_MEMPRO eq 'True');
     } else {
	print OPT "-mempro " if($RHAP_MEMPRO eq 'True');
	print OPT "-perfpro " if($RHAP_PERFPRO eq 'True');
	print OPT "-trace " if($RHAP_TRACE eq 'True');
	print OPT $RHAP_COV,"\n";
	print OPT "-noinstrdir=\"",eval($RHAP_LIBS),"\"\n" if($RHAP_LIBS ne '');
     }
     print OPT "-ignore=TestRealTime.cpp,TestRealTime.c\n";
     print OPT $RHAP_USEROPTIONS, "\n" if($RHAP_USEROPTIONS  ne '');
     print OPT $RHAP_USERSETTINGS,"\n" if($RHAP_USERSETTINGS ne '');
     close OPT;
     print "$pwd/$optionFile generated\n"
   } else {print "unable to write $optionFile\n";}
}

#print "-- $CC --$CPP --$LINK_CMD\n";
unless($CC){
   print "try getting compiler and linker from TDP...\n"; #from tp.ini and tpcpp.ini
   if(defined($ENV{'ATLTGT'}) && (-d $ENV{'ATLTGT'})){
	if(open INI,"$ENV{'ATLTGT'}/tpcpp.ini"){
	 while(<INI>){
		if(/^COMPILER=\"(.*)\"/) {$CPP=$1 unless($CPP);}
		if(/^LINKER=\"(.*)\"/) {$LINK_CMD=$1 unless($LINK_CMD);}
	 }
	close INI;
	}
	if(open INI,"$ENV{'ATLTGT'}/tp.ini"){
	 while(<INI>){
		if(/^COMPILER=\"(.*)\"/) {$CC=$1;}
		if(/^LINKER=\"(.*)\"/) {$LINK_CMD=$1 unless($LINK_CMD);}
	 }
	close INI;
	}
   } else {
	print "ATLTGT Not defined !!!\n" unless(defined($ENV{'ATLTGT'}));
	print "TDP $ENV{'ATLTGT'} does not exist. Check TestRT installation\n";
   }
}
$CPP=$CC unless($CPP);
$LINK_CMD=$CC unless($LINK_CMD);
print "found compiler as CC=$CC CPP=$CPP Linker=$LINK_CMD\n";

#write the TestRT.bat or TestRT.sh according to -bat/-sh option
my $batext="bat";
my $set="set";
my $prefix='%';
if($batType=~m/^-sh/){
$batext="sh";
$set="";
$prefix='$';
}

if($CC ne "") {
   if(open BAT, ">TestRT.$batext") {
	$CC=$CPP if($RHAP_LANG eq 'CPP');
	print BAT "$set CC_VAR=$CC_VAR\n";
	print BAT "$set CPP_VAR=$CPP_VAR\n";
	print BAT "$set LINK_CMD_VAR=$LINK_CMD_VAR\n";
	print BAT "$set CC=$CC\n";
	print BAT "$set CPP=$CPP\n";
	print BAT "$set LINK_CMD=$LINK_CMD\n";
	print BAT "$set BATCH_CMD=$BATCH_CMD\n";
	print BAT "$set ATLTGT=$ENV{'ATLTGT'}\n";
	print BAT "$set RTRTMEMPRO=$RHAP_MEMPRO\n";
	close BAT;
   } else {print "unable to write TestRT.$batext\n";}
} else {print "unable to write TestRT.$batext because CC not defined\n";}

#check the build batch file
if(open(MAK,$BATCH_CMD)){
    while(<MAK>){
	if(/make/ && /\Q$prefix\E1/){
	    unless(/\Q$prefix\E9/){
		print "WARNNING: the file $BATCH_CMD may not be completed\n";
		print "make sur ${prefix}3 to ${prefix}9 is on the line:\n$_\n";
	    }
	    last;
	}
    }
    close MAK;
}

#check used variables in the Makefile
if($RHAP_MAKEFILE ne '' && open(MAK,$RHAP_MAKEFILE)){
    my $foundLD=0;
    my $foundCC=0;
    while(<MAK>){
	if(/\Q$LINK_CMD_VAR\E/){
	    $foundLD=1;
	}
	if(/\Q$CC_VAR\E/){
	    $foundCC=1;
	}
	if(/\Q$CPP_VAR\E/){
	    $foundCC=1;
	}
	last if($foundCC && foundLD);
    }
    close MAK;
    print "WARNNING: The makefile '$RHAP_MAKEFILE' does not use '$LINK_CMD_VAR' for link command\n" unless($foundLD);
    print "WARNNING: The makefile '$RHAP_MAKEFILE' does not use '$CC_VAR' nor '$CPP_VAR' for compilation command\n" unless($foundCC);
    print "WARNNING: 	change the 'MakeFileContent' content to use appropriate variable in Rhapsody project properties, or\n			change/add LINK_CMD_VAR=ld and CC_VAR=cc and CPP_VAR=cc in $eqFile to specify the used variable name in makefile\n" unless($foundCC && $foundLD);
} else {print "WARNNING: Unable to open makefile '$RHAP_MAKEFILE' for checks\n";}

sub readEqLine {
my $values=shift;
my $tgt;
    	if($values=~/ATLTGT=(\")?(.+)\1/){
		$tgt=$2;
	} elsif($values=~/ATLTGT=([^\s]+)(\s)?/){
		$tgt=$1;
	}
	if($tgt) {
		unless($tgt=~m/[\\\/]/){
			my $xdpName=$tgt;
			$tgt=getAppData().'/targets/'.$tgt;
			$tgt=$TESTRTDIR  .'/targets/'.$xdpName unless(-e $tgt."/tp.ini");
		}
		$ENV{'ATLTGT'}=$tgt;
		print "setting env ATLTGT=$tgt\n";
		print "TDP does not exist. Check TestRT installation\n" unless(-d $tgt);
    	}
    	if($values=~/CC=\"(.+)\"/){
		$CC=$1;
    	}
    	elsif($values=~/CC=([^\s]+)(\s)?/){
		$CC=$1;
    	}
    	if($values=~/CPP=\"(.+)\"/){
		$CPP=$1;
    	}
    	elsif($values=~/CPP=([^\s]+)(\s)?/){
		$CPP=$1;
    	}
    	if($values=~/LINK_CMD=\"(.+)\"/){
		$LINK_CMD=$1;
    	}
    	elsif($values=~/LINK_CMD=([^\s]+)(\s)?/){
		$LINK_CMD=$1;
    	}
    	if($values=~/BATCH=([^\s]+)(\s)?/){
		$BATCH_CMD=$1;
    	}
    	if($values=~/CPP_VAR=([^\s]+)/){
		$CPP_VAR=$1;
    	}
    	if($values=~/CC_VAR=([^\s]+)/){
		$CC_VAR=$1;
    	}
    	if($values=~/LINK_CMD_VAR=([^\s]+)/){
		$LINK_CMD_VAR=$1;
    	}
}
exit(0);
1;