##########################################################################
#                                                                        # #
# Licensed Materials - Property of HCL                                   #   #
# Restricted Material of HCL                                             #######
# HCL OneTest Embedded                                                         #
# � Copyright HCL Technologies. 2020 All Rights Reserved.                      #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with HCL Technologies.    #
#                                                                              #
################################################################################
###### specific post processing steps for model colorization #############

my $rtistCovColorModelVerbose=0;

# Warning : do not rename or change this prototype, it overwrites placeholder from Tools.pl
sub modelPost {
    $ENV{ATTOLSTUDIO_VERBOSE}=1;
    my $colorFile=$RTIST_TARGET_COLORIZATION;   #generated file
	my $modelFDCdir=$RTIST_MODEL_FDC_DIR; #diretory where are the FDC model files 
    my @paramFDC;
    my %fdcDirList;$fdcDirList{$RTIST_TARGET_BUILD}=0;
    my @emxFiles;
    push(@emxFiles,(split(/,/,$RTIST_EMX_FILES)));
    my $tcjsFile=$RTIST_APP_TCJS_FILE;
    #check for multiple projects build (1 executable on top of libraries)
#    print "#>>LOGFILENAME=$LOGFILENAME\n";
    if(open(LOG,$LOGFILENAME)){
        my $dir;
        while(<LOG>){
#            print;
            if(/^COR\s(.*)$/){
                $colorFile=$1;
#                print "colorFile $colorFile\n";
            }
            elsif(/^TCJ\s(.*)$/){
                $tcjsFile=$1;
#                print "#LOG tcjsFile $tcjsFile\n";
            }
            elsif(/^DIR\s(.*)$/){
                $dir=$1;
#                print "look at DIR $dir\n";
            }
            elsif(/^EMX\s(.*)$/){
                push(@emxFiles,(split(/,/,$1)));
#                print "look at $1\n";
            }
            elsif(/^FDC\s(.*)$/){
                my ($fdcdir,$f,$e)=SplitFileName(absolutePath($1,$dir));
                $fdcdir=~s,\\,\/,g;
                $fdcdir=~s,\/$,,;
                $fdcDirList{$fdcdir}++;
#                print "fdcdir $fdcdir\n";
            }
        }
        close LOG;
    } else {
        print #OTE xtp file not found $LOGFILENAME\n";
    }
    # store model coverage files
    mkdir $modelFDCdir unless(-d $modelFDCdir);
    #filter emx files
    my @emxFileList;my $prev;
    foreach my $emx (sort @emxFiles){
        push(@emxFileList,$emx)if($emx ne $prev);
        $prev=$emx;
    }
    my @RTistFdcConvLog;
    foreach my $fdcdir (keys %fdcDirList){
        my $status = SystemP("RTistFdcConv \"$fdcdir\" \"$modelFDCdir\" \"".join(',',@emxFileList)."\"" );
        #return code = number of converted FDC 
        push(@RTistFdcConvLog,$fdcdir.'/rtcovstat.cov');
    }
#    print "##OTE ",join(',',@RTistFdcConvLog),"\n";
    updateFDCinLog(\@RTistFdcConvLog);
    unlink(@RTistFdcConvLog);
	opendir(DIR,"$modelFDCdir");
    my @dirContents = readdir(DIR);
    closedir(DIR);
	foreach my $file (@dirContents) {
		push(@paramFDC,"\"".$modelFDCdir.'/'.$file."\"") if($file =~m/\.fdc$/i);
	}
	my $command="attolcov ".join(' ',@paramFDC,"\"".$ATC_TIO_NAME."\"")." -block -output=model_cov.txt >>modelCov.log";
	$status=SystemP($command) if(@paramFDC);
    my $relative_tcjs=$tcjsFile;
    my $relativeworkspace="platform:/resource";
    $relative_tcjs=~s/$WORKSPACE_PATH/$relativeworkspace/e;
    $relative_tcjs=~s/ /%20/g;
	if($status==0){
		if(open(IN,"model_cov.txt")){
			open(OU,">$colorFile");
            print OU "TC_URI $relative_tcjs\n"; 
            print &PerlTranslate("rtistCovColorModel.pl", "#OTE-color Report Generation $colorFile...\n");
#            print "generating  for $relative_tcjs\n";
			my $found='';
			my $result='';
			my $count=' ';
			my $lookingForCount=0;
			while(<IN>) {
				if (/^\s*(.*\.(emx|efx)#.*)\s*:$/){
                   $found=$1;
                   $found=~s/%2520/%20/g; # BoZ : temporary hack until fdcconv is fixed
                }
				elsif($lookingForCount){#find the count number on line following 'Statement block'
					$lookingForCount=0;
					$count='';
					#             /, line 1, 2 time(s)
					if(/^\s*\/,\sline\s\d,\s(\d+)\stime/){
						$count=$1.' ';
					}
					print OU $result, $count, $found, "\n"; 
 					print $result, $count, $found, "\n" if($rtistCovColorModelVerbose); 
					$found='';
				}
				elsif (/^\s*Statement blocks\s/ && ($found ne '')) # this line should immediately follow emx ref
				{
					$result='UNCOVERED ';
					if (/^\s*Statement blocks\s*\.+\s*\d+\.\d\%\s*\((\d+)\/(\d+)\)/) { #print;
						if($1 != 0) {
							$result= 'COVERED ';
							$lookingForCount=1;#find the count number if any, on line following 'Statement block' then print result
						}
						else
						{
							print OU $result, $found, "\n"; 
							print $result, $found, "\n" if($rtistCovColorModelVerbose); 
							$found='';
						}
					}
				} 
				else
				{
					$found='';
				}
			} 
            #clean up (close handlers / delete temp files)
            close IN; 
            close OU;
            RegisterGenFiles("COR $colorFile"); 
			unlink("model_cov.txt","modelCov.log");
		}
	}

    if ($status != 0 ) { 
        print &PerlTranslate("rtistCovColorModel.pl", "Failed.\n"); 
    } else { 
        print &PerlTranslate("rtistCovColorModel.pl", "Done.\n");
    }
    return $status;   
}

#suppress FDC code from LOGFILENAME and add FDC model instead
#regenerate @FDCFILES for postprocessing
sub updateFDCinLog
{
 my $convlogFiles=shift;
 my @fdcModel;
 if(open(OU, ">$LOGFILENAME.tmp")){
  if(open(IN, $LOGFILENAME)) {
    while(<IN>){
        my $l=$_;
        next if($l=~m/^FDC\s/ && $l!~/\.MDL\.fdc/i);
        push(@fdcModel,$1) if(/^FDC\s(.+)$/);
        print OU;
    }
    close IN;
#    print "#OTE updateFDCinLog\n";
    foreach my $convlog (@$convlogFiles){
#    print "#OTE update with $convlog\n";
        if(open(IN, $convlog)) {
#            print "#OTE $convlog\n";
            while(<IN>){
                print;
                next if(/^#/);
                push(@fdcModel,$1) if(/^FDC\s(.+)$/);
                print OU;
            }
            close IN;
        }
    }
    close OU;
    rename($LOGFILENAME.'.tmp',$LOGFILENAME);
    @FDCFILES = @fdcModel if(@fdcModel);
  } else
  {  
    close OU;
    unlink($LOGFILENAME.'.tmp');
  }
 }
}
1;

