#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  Copyright HCL Technologies Ltd. 2016, 2020. All Rights Reserved.
#  Copyright IBM Corporation 1999, 2016. All Rights Reserved.
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                 -*-  Rtist compiler/linker prefix  -*-                       #
#                          Rtist build instrumentor                            #
#                                                                              #
################################################################################
#
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

## globals ############
our $TestRTCC=1;

# Miscellaneous functions
use File::Copy;
use File::Spec;
die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR=$ENV{TESTRTDIR};
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "instrum.pl";
require $SCRIPTDIR . "BatchCCDefaults.pl";

my $VERBOSE_MODE=$ENV{ATTOLSTUDIO_VERBOSE};

my $RTIST_TARGET_PROJECT;
my $RTIST_TOP_CAPSULE_BASENAME='TOP'; #default value if not found
my $RTIST_OTE_PROJECT = '';
my $RTIST_PROJECT_ISLIB = 0; #0 means this is the main project
my $overwiteParams=0;
my $projectEnvFile;
my @tcjs_sources;
#our $ATC_INFORMATION;


sub findAppTcjs {
	my $mkNumFile = 0;
    my $appfilename = "";
	while(!-e "$mkNumFile.mk" && $mkNumFile<20) {$mkNumFile++;}

    open(my $fh, '<:encoding(UTF-8)', "$mkNumFile.mk") or die "Could not open file '$mkNumFile.mk' $!";
 
    while (my $line = <$fh>) {
        chomp $line;
        if ($line =~ m/^# Generated from (.*)/) {
            $appfilename = $1;
			$appfilename =~s/\\/\//g;
            last;
        }
    }
    close ($fh);
    print STDERR "appfilename = $appfilename\n" if ($VERBOSE_MODE);
    return $appfilename;
}

sub findRTS_HOME {
    my $mkfile = "Makefile";
    my $RTS_HOME = "";
    open(my $fh, '<:encoding(UTF-8)', $mkfile) or die "Could not open file '$mkfile' $!";
    while (my $line = <$fh>) {
        chomp $line;
        if ($line =~ m/^RTS_HOME\s*=\s*(.*)\s*$/) {
            $RTS_HOME = $1;
			$RTS_HOME =~s/\\/\//g;
            last;
        }
    }
    close ($fh);
    print STDERR "RTS_HOME = $RTS_HOME\n" if ($VERBOSE_MODE);
    return $RTS_HOME;    
}


sub analyseTcjsFile {
    my $RTIST_APP_TCJS = shift;
    return if ($RTIST_APP_TCJS eq '');
    if (-e $RTIST_APP_TCJS) {
        open(TCJS, '<:encoding(UTF-8)', $RTIST_APP_TCJS) or die "Could not open file '$RTIST_APP_TCJS' $!";
		my $getSources=0;
        while (<TCJS>) {
            chomp ;
			if (/^\s*tc.sources\s*=\s*\[(.*)$/){
				push(@tcjs_sources,$1) if($1 ne '');
				$getSources=1;
			}
			elsif($getSources) {
				if(/^\];/){
					$getSources=0;
				} else {
					push(@tcjs_sources,$_);
				}
			}
			elsif (/^\s*tc.targetProject\s*=\s*'\/(.*)'\s*;/) {
                $RTIST_TARGET_PROJECT = $1 unless($overwiteParams & 2);
            }
            elsif (/^\s*tc.type\s*=/ && /\.Library\s*;/) {
                $RTIST_PROJECT_ISLIB = 1; #This is not the main project
            }
            elsif (/^\s*tc.instrumentationData\s*=\s*{\s*project:\s*'(.*)'(.*)\s*}\s*;/) {
                $RTIST_OTE_PROJECT = $1 unless($overwiteParams & 1);
            }
            elsif (/^\s*tc.topCapsule\s*=\s*('platform:\s*\/resource\/(.*)\/(\w*).e[fm]x)#/) {
                $RTIST_TOP_CAPSULE_BASENAME = $3 unless($overwiteParams & 4);
				unshift(@tcjs_sources,$1);
            }                                
        }
        close (TCJS);
    } else {
        print STDERR "app.tcjs not found : $RTIST_APP_TCJS\n";
    }
}

sub generateLocalEnvFileAndClean {
    # generate a local "envTestRTcc" that reference global one and override some stuff
    # do not generate if it already exists
    my $envFile = shift;
 
        # clean up
        my $pwd=GetCurrentDir();
        $pwd=~ s/\\/\//g;
        # overwite tcjs values with parameters
        # possible args :
        # -TARGETPROJECT='<value>'
        #       ex : -TARGETPROJECT='/TrafficLight target with space'
        #       (starts with a '/' as in tcjs file)
        # -OTEPROJECT='<value>'
        #       ex : -OTEPROJECT='my OTE with spaces'
        #       (does not starts with a '/' as in tcjs file instrumentationData)
        # -TOPCAPSULEBASENAME='<value>'
        #       ex : -TOPCAPSULEBASENAME='TrafficLightComponent'
        my $targetOTEProjectIsFullPath=0;
        while($ARGV[0]=~m/^"?-/)  {
			my $arg =shift( @ARGV );
            if ($arg =~/^"?-TARGETPROJECT='?\/?([^'"]*)\['"]*/) {
                $RTIST_TARGET_PROJECT = $1;
                $overwiteParams |=2;
                if ($arg =~/^"/) {
                    # path is directly a full path, and so, do not use map to retrieve it
                    $targetOTEProjectIsFullPath=1;
                }
                print STDERR "#OTE arg target project: $RTIST_TARGET_PROJECT\n";
            }
            elsif ($arg =~/^-OTEPROJECT=([\"']?)(.*)\1/) {
                $RTIST_OTE_PROJECT = $2;
                $overwiteParams |=1;
                print STDERR "#OTE arg project: $RTIST_OTE_PROJECT\n";
            }
            elsif ($arg =~/^-TOPCAPSULEBASENAME=?'(.*)?'/) {
                $RTIST_TOP_CAPSULE_BASENAME = $1;
                $overwiteParams |=4;
                print STDERR "#OTE arg topcapsule basename: $RTIST_TOP_CAPSULE_BASENAME\n";
            }
            else {
                print STDERR "#OTE unknown arg $arg\n";
            }
        }
        # find and analyze tcjs file
        $RTIST_APP_TCJS = findAppTcjs();
        analyseTcjsFile($RTIST_APP_TCJS);
                
        if (defined $RTIST_OTE_PROJECT) {
            my $projectDir=$RTIST_OTE_PROJECT;
            unless($RTIST_OTE_PROJECT=~m/^(\w:)?[\\\/]/) #path to OTE project NOT known
            {
                # load general environment file to retrieve eclipse project map and verbose mode
                # since envFile in userProfile may come from an other projet
                # we use it only to get ECLIPSE_PROJECT_MAP giving path to real location of project
                # apropriate envFile will be loaded bellow when real location will be known
                # 
                my $projectEnvFile = $ENV{'USERPROFILE'}."/envTestRTcc.pl";
                unless(-e $projectEnvFile) {
                    if($^O =~ /win32/i){
                        $projectEnvFile = $ENV{HOMEDRIVE}.$ENV{HOMEPATH}."/envTestRTcc.pl";
                    } else {
                        $projectEnvFile = $ENV{HOME}.'/envTestRTcc.pl';
                    }
                }
                $projectEnvFile=~ s/\\/\//g;
                if (-e $projectEnvFile) {
                    require $projectEnvFile;
                    $VERBOSE_MODE=$ENV{ATTOLSTUDIO_VERBOSE};
                } else {
                    die "$projectEnvFile not found: create a OneTestEmbedded project and save settings before compiling...\n";
                }
                if ($VERBOSE_MODE) {
                    print STDERR "#OTE-MAP found in $projectEnvFile\n";
                    while( my ($k,$v) = each(%ECLIPSE_PROJECT_MAP) ) {
                        print STDERR "#OTE-INFO# project $k location:$v\n";
                    }
                }
                if (exists( $ECLIPSE_PROJECT_MAP{$RTIST_OTE_PROJECT} )){
                    $projectDir = $ECLIPSE_PROJECT_MAP{$RTIST_OTE_PROJECT} ;
                } else {
                    #try to find OTE project from current dir
                    my $curdir=$pwd;
                    while($curdir ne '' && !(-e "$curdir/$projectDir/build/envTestRTcc.pl")) {
                        $curdir=~s,(\w:)?\/[^\/]+$,,;
#                        print "### try with $curdir/$projectDir/build \n";
                    }
                    $projectDir="$curdir/$projectDir" if(-e "$curdir/$projectDir/build/envTestRTcc.pl");
                }
                print STDERR "#OTE-PRJ $RTIST_OTE_PROJECT found at: $projectDir\n";
            }
            $projectDir =~ s/\\/\//g;
            if(opendir(DIR,$projectDir)) {
                my @dirContents = readdir(DIR);
                closedir(DIR);
                my $envFileDate=0;
                foreach my $subdir (@dirContents) {
                    if(-d $projectDir.'/'.$subdir){
                        my $file=$projectDir.'/'.$subdir.'/envTestRTcc.pl';
                        my $dof=GetModificationTime($file);
                        if($dof > $envFileDate) {
                            $projectEnvFile=$file;
                            $envFileDate=$dof;
                        }
                    }
                }
            }
            die " envTestRTcc.pl not found in project $projectDir: create a OneTestEmbedded project and save settings before compiling...\n" unless(($projectEnvFile ne '') && (-e $projectEnvFile));
            print STDERR "#OTE-settings $projectEnvFile ===\n" if ($VERBOSE_MODE);
        } else {
            die " OTE project not found: create a OneTestEmbedded project and save settings before compiling...\n";
        }
        
        copy($projectEnvFile, $envFile);
		require $envFile;
        # ok, we have good one, let's reload it
        my $project_location=$WORKSPACE_PATH.'/'.$PROJECT_NAME;
        
        if (exists( $ECLIPSE_PROJECT_MAP{$PROJECT_NAME} )) {
            $project_location=$ECLIPSE_PROJECT_MAP{$PROJECT_NAME};
        }
        if ($targetOTEProjectIsFullPath == 0) {
            if (exists( $ECLIPSE_PROJECT_MAP{$RTIST_TARGET_PROJECT} )) {
                $RTIST_TARGET_PROJECT=$ECLIPSE_PROJECT_MAP{$RTIST_TARGET_PROJECT};
            } else {
                $RTIST_TARGET_PROJECT=$WORKSPACE_PATH."/".$RTIST_TARGET_PROJECT;
            }
        }
        # remove old logfile and out_file : if we recompile, it's obsolete
        $LOGFILENAME=$project_location.'/'.$ECLIPSECONFNAME.'/'.$LOGFILENAME;
        $ATL_OUT_FILE=$project_location.'/'.$ECLIPSECONFNAME.'/'.$ATL_OUT_FILE unless($ATL_OUT_FILE=~m/^(\w:)?[\\\/]/);
        $ATL_OUT_FILE=~ s/\\/\//g;
        #LOGFILENAME cannot be removed on each build because of multi-build (libs then EXE) must use same LOGFILENAME
        #unlink($LOGFILENAME) if (-e $LOGFILENAME);
        unlink($ATL_OUT_FILE) if (-e $ATL_OUT_FILE);

        my $buildTime=GetDate();
        RegisterGenFiles("DTT $buildTime","DIR $pwd","ENV ".absolutePath($envFile));
        # the goal is to get only one RTIST_TARGET_COLORIZATION when reading all envFiles in multi-projects
        RegisterGenFiles("COR $RTIST_TARGET_PROJECT/$RTIST_TOP_CAPSULE_BASENAME.mcov","TCJ $RTIST_APP_TCJS") unless($RTIST_PROJECT_ISLIB);
        
        # modification for this shot
        $INTDIR='.';
		$OUTDIR='.';
		$ENV{ATLTGT} = $TARGETDIR;
		my $RTS_HOME = findRTS_HOME();
		print "#OTE TARGETDIR: $TARGETDIR\n";
		push(@ATG_EXCLUDED_DIRS , $RTS_HOME);

        print STDERR "### generating $pwd/$envFile\n" if ($VERBOSE_MODE);

        # generation for next times
        open(FH, '>>', $envFile) or die $!;
        
        print FH "# local overloading for Rtist environment\n";
        print FH "\$NOSTOP=1;\n"; #force to continue to build despite instrum errors
        print FH "\$ATL_SIGNAL_DUMP=2+((8192+2)*256);\n"; #force dump on first Ctrl+C
        print FH "\$INTDIR='.';\n";
        print FH "\$OUTDIR='.';\n";        
        print FH "\$LOGFILENAME='$LOGFILENAME';\n";
        print FH "\$ATL_OUT_FILE='$ATL_OUT_FILE';\n";
        print FH "\$MODEL_POST_SCRIPT='rtistCovColorModel.pl';\n";
        print FH "\$XTPFILE=\$LOGFILENAME;\n";
        print FH "\$RTIST_APP_TCJS_FILE=\"$RTIST_APP_TCJS\";\n"; #8.3.0.1  only   useless in 8.3.1  
        print FH "\$RTIST_TARGET_BUILD='$pwd';\n"; 
        print FH "\$RTIST_MODEL_FDC_DIR=\$RTIST_TARGET_BUILD.'/mdl_fdc';\n";# used to store modified fdc
        print FH "\$RTIST_TARGET_PROJECT='$RTIST_TARGET_PROJECT';\n";
        print FH "\$RTIST_TARGET_COLORIZATION=\$RTIST_TARGET_PROJECT.'/$RTIST_TOP_CAPSULE_BASENAME.mcov';\n";
        print FH "push \@ATG_EXCLUDED_DIRS , '$RTS_HOME';\n";
        print FH "\$ENV{'ATLTGT'} = \$TARGETDIR;\n";
		
        print FH "\$RTIST_EMX_FILES='";
		my $coma='';
        my %EMX;
        my $RTIST_EMX_FILES;
		foreach my $s (@tcjs_sources){
			$s=~s!\%20! !g;
			$s=~s!\#[^']*'\s*[,;]*\s*$!!;
			$s=~s!^\s*'platform:\/resource\/([^\/]+)!$1!;
#           print STDERR "emx1 : $s\n";
			my $prj=$1;
			$s=~s!^$prj!$ECLIPSE_PROJECT_MAP{$prj}! if(defined $ECLIPSE_PROJECT_MAP{$prj});
#           print STDERR "emx2 : $s\n";
            unless(defined $EMX{$s}){
               $EMX{$s}=1;
               unless(-e $s){
                    if(-e $WORKSPACE_PATH.'/'.$s){
                        $s=$WORKSPACE_PATH.'/'.$s;
                    }
                    else
                    {
                        print STDERR "### ERROR absolute project path for $prj not found for emx $s. Edit OTE settings may help!!!\n";
                    }
                }
                print STDERR "emx : $s\n" if ($VERBOSE_MODE);
                $RTIST_EMX_FILES.="$coma$s";
                print FH "$coma$s";$coma=',';
            }
		}
		print FH "';\n";
        RegisterGenFiles('EMX '.$RTIST_EMX_FILES);
        print FH "1;\n";
        
        close(FH);
        # check if hitcount is plugged or not
        unless ($ATC_INFORMATION == 2) {
            # hit count is not present
            print "Warning : Hit count for model colorization is not active\n";
            print "\tto active it, open OTE build settings, Code coverage => Instrumentation control\n";
            print "\tand change Information mode to \"report hit count\"\n";
        }

}

my $localTestrtEnvFile = 'envTestRTRtistcc.pl';
if(-e $localTestrtEnvFile) {
	require $localTestrtEnvFile;
} else {
	generateLocalEnvFileAndClean($localTestrtEnvFile);
}

#remove any options found before compiler name
shift(@ARGV) while($ARGV[0]=~m/^\"?-/);
$CFLAGS=1 if($USE_CFLAGS_TP);
our $instrumOpt = BuildCCppInstrOptions($TDP_LANGUAGE,"","","");

if(-f "$TARGETDIR/cmd/attolcc.pl") {
    require $TARGETDIR . "/cmd/attolcc.pl";
} else {
	die   "#?$SMALL_PRODUCT_NAME-I-".uc($launcher).",ATLTGT=$TARGETDIR is not a TDP directory.\n\n" unless(-f "$TARGETDIR/tp.ini" || -f "$TARGETDIR/tpcpp.ini");
	print "#?$SMALL_PRODUCT_NAME-I-".uc($launcher).",attolcc.pl not found in this TDP. attolcc for gcc is used instead.\n\n";
	require $SCRIPTDIR."attolcc_gcc.pl";
}

 
if($main::TestRTCC ==2){
    # clean log
    #
   print "#OTE-LINK Done\n";
}
exit 0;
