##############################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
##############################################################################
#
# Author: Simon Washbrook
#
# Development notes:
# Option -html not implemented as it does not work
#
##############################################################################


#use strict;
#use TestRTUtil;
use FileHandle ;
use File::Spec;
BEGIN {
  if ($^O eq "MSWin32") {
   use File::DosGlob 'glob';
 }
}
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$ATT_SPLIT_SIZE=5000;
$ATG_MODULE_AS=0;
$TESTRTDIR=$ENV{'TESTRTDIR'};
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";
# This script depends on the followings (both studio'n Target Package)
require $SCRIPTDIR . "Fcts.pl";
$BINDIR=GetBinDir();
setMachineVars();
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";
require $SCRIPTDIR . "instrPostpro.pl";

$ATKMODE=1;
$ATCMODE=1;
$ATTMODE=1;
$ATPMODE=1;
$ATQMODE=1;
$ATAMODE=1;
$ATZMODE=1;
$ATGMODE=1;
$DCPMODE=1;
@TSFFILES=();
@FDCFILES=();
$RESULT_EXT=''; #result ext to be overwrote by the decode.pl to take into account these file types
my $clean=0;
my $theSptFile=''; #non existing spt file when onTheFly
my $nodeNumber;
my $instOpt;
my $run="-view";
my $dirtp=".";
my $curdir=GetCurrentDir();
my $defaultRtpName="TestRTreports";
my $projectName=$defaultRtpName;
my $filter;
my $envVariables="RTRT_EXTERNAL_EXEC=1";
my %args=("NAME"=>"Results",verbose=>0);
my (@xrd,@rtx,@crc,@met,@tio,@tpf,@tqf,@tdf,@tgf,@ccf,@dcp,@tzf,@tgx,@dcx,@tzx,@tqx,@spt, @studio_args);
$LOGFILENAME="studioreport.log";
@existed = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0);# see "Kinds of output files" into Split.pl

##############################################################################
# Help

sub help {
  my ($d,$n,$e) = SplitFileName($0);
  print "$n : Creates a temporary project with the test result files passed asparameter.\nThese results can then be exported to HTML\n";
  print "Command line:\n";
  print "\t$n [-help] [-html <dir>] [-keep] [-clean[All]] [-verbose] [<report files>]\n";
  print "Options:\n";
  print "\t-help Displays this help message\n";
  print "\t-html <dir> create the directory for HTML exports\n";
  print "\t-keep Keeps the temporary project that was generated\n";
  print "\t-clean remove all dynamic results, -cleanAll remove all report files\n";
  print "\t-verbose show all files given as studio parameters\n";
  print "\t+filter when the spt file data require filtering using the TDP decode.pl\n";
  print "\tlist of space seperated file names. Only test result files are accepted.\n";
  print "Report files must have the following extensions\n";
  print "\t.spt        global result file generated by the execution\n";
  print "\t.xrd        Component Testing report file\n";
  print "\t.rtx        Graphical report file\n";
  print "\t.crc        Rule Checker report file\n";
  print "\t.met        Static Metric report file\n";
  print "\t.fdc, .tio  Code Coverage report files\n";
  print "\t.tsf, .tpf  Memory Profiling report files\n";
  print "\t.tsf, .tqf  Performance profiling report files\n";
  print "\t.tsf, .tdf  Runtime Tracing report file\n";
  print "\t.tsf, .tgf  Ctrl Coupling report file\n";
  print "\t.fdc, .dcp  Data Coupling report files\n";
  print "\t.fdc, .ccf  Ctrl Coupling report file\n";
  print "\t.tsf, .tzf  Stack size report file\n";
  print "\t.log, .xtp  file providing the listing of files like attolccReport.xtp\n";
  print "Files must have absolute paths or be relative to the current directory.\n";
  print "\n\n(c) Copyright 2005-2018 IBM Corp. All right reserved\n";
  sleep(15);
}
###########################################################################
# end called at the end of the instrumentation (link time)
sub reports
{
  my $log=shift;

	my $Opt;
	my $otsfiles;
	my $file;
	my ($xdir,$xn,$xe)=SplitFileName($log);
	my $filedir=absolutePath($xdir,$curdir);
	print "report dir:$xdir,$curdir\n" if ($args{verbose});
	if($projectName eq $defaultRtpName) {
		$projectName = $xn ;
		$dirtp=$filedir;
		$dirtp=~ s,[\\\/]$,,;
	}
	print "read file:$filedir/$xn.$xe\n" if ($args{verbose}>1);
	open LOG, "<$filedir/$xn.$xe";
	while(<LOG>)
	{
	    chomp;
	    print "line:$_\n" if ($args{verbose}>1);
	    if (/^#OPT\s+A([AGDZPQqCTf]*)\s+\"([^\"]*)\"/){
	    	#instrum options
	    	$Opt=$1;
	    	$instOpt=$Opt;
	    	if($2){
	    		$args{NAME}=$2 ;
	    		$args{NAME}.='_' if(-f $2);
	    	}
			$ATCMODE=0 unless($instOpt =~/C/);
			$ATTMODE=0 unless($instOpt =~/T/);
			$ATPMODE=0 unless($instOpt =~/P/);
			$ATQMODE=0 unless($instOpt =~/Q/);
			$ATAMODE=0 unless($instOpt =~/A/);
			$ATZMODE=0 unless($instOpt =~/Z/);
			$ATGMODE=0 unless($instOpt =~/G/);
			$DCPMODE=0 unless($instOpt =~/D/);
			$ATQ_COMPUTE_MINMAX=2 if($instOpt =~/q/);
	    }
	    elsif (/^#TGT\s+(.+)$/){#recover ATLTGT
	    	$ENV{ATLTGT}=$1 unless($ENV{ATLTGT});
	    }
	    elsif (/^[#;]/){#line ignored
	    }
	    elsif(/^([A-Z]{3}) (.+)$/){
		my $c=$1;
		$file=$2;
		next unless($c=~ m/^(OTS|DUM|SPT|LOG|XTP|XRD|CRC|FDC|TIO|TSF|TPF|TQF|TDF|TGF|TZF|DCP|CCF|MET|RTX)/);
		$f=absolutePath($file,$filedir);

		if($c eq "OTS"){
		  $otsfiles.=$f.' ';
		}elsif ($c eq "DUM"){
	  	  unlink($f);
		}elsif ($c eq "SPT"){
		  my @l=grep(/^\Q$f\E$/, @spt);
		  unless(scalar(@l)) {push(@ARGV,$f);}
		  if(($Opt=~ m/f/)&& !(-f $f))  {
			$theSptFile="\"$f\"" unless($theSptFile);
		  }
		}else{
			  print $f;
		  my @l=grep(/^\Q$f\E$/, @ARGV);
			  if(scalar(@l)){
				print ' duplicate ignored' ;
			  } else {
				push(@ARGV,$f);
			  }
			  print "\n";
		}
	    }
	    elsif(/\w\.(\w\w\w)\"?\s*$/){
	    	  s,\",,g;
		  s,\s+$,,;
	    	  $f=$_;
		  $f=absolutePath($f,$filedir) unless(-f $f);
		  my @l=grep(/^\Q$f\E$/, @ARGV);
		  push(@ARGV,$f) unless(scalar(@l)); # this is not a log generated by us
		}
	}
	close(LOG);
	if($clean >1){
	   unlink($log);
	}else{
	   if($otsfiles){
		my $command="atopospro -ots $otsfiles -tdf atltrace.tdf -xrd contract.xrd";
		print "Running the command \n$command\n" ;
		system ($command);
		push(@ARGV,"contract.xrd");
	   }
	}
}

##############################################################################
# Parse parameters to extract report file types

sub parseArgs {
my $nbArgs=scalar(@ARGV);
my $reportFound=0;
  unless ($nbArgs) {
  	print "looking for sumary file\n";
  	$reportFound=1;
	if(-f "attolccfiles.log")
	{
		reports("attolccfiles.log");
	} elsif(-f "TestRTccfiles.log") {
		reports("TestRTccfiles.log");
	} elsif(-f "TestRTccfiles.xtp") {
		push(@ARGV,"TestRTccfiles.xtp");
	} elsif(-f "attolccReport.xtp") {
		push(@ARGV,"attolccReport.xtp");
	} else {
    		print "\nERROR: No arguments\n";
    		help();
    		exit 1;
	}
  }

  my @unknownFiles;
  my $lmth;
  while (@ARGV) {
    $_ = shift @ARGV;
    s/\"//g;
    print "arg: $_\n" if ($args{verbose});
    if (/\*/) {
      print "\nExpanding argument:\n$_\n" if ($args{verbose});
      #unshift(@studio_args, "\"".$_."\"");
      $_ =~ s,\\,/,g;
      $_ =~ s/(\s)/\\$1/g;
      my @files = glob $_;
      if ($args{verbose}) {print join(',',@file), "\n";}
      unshift(@ARGV,@files);

    } elsif (/^-help$/) {
      help();
      exit 0;
     } elsif (/^-html$/) {
       my $lmth=1; #my
       $envVariables=''; #remove blocking exec
       $args{html} = shift @ARGV;
       push(@studio_args, "-html $args{html}");
       unless (-d $args{html}) {
	mkdir ($args{html},0777) || die "Can not create the html output directory \"".$args{html}."\"\n";
       }
    } elsif (/^-clean$/i) {
      $clean = 1;
    } elsif (/^-cleanall$/i) {
      $clean = 2;
    } elsif (/^-view$/) {
      $args{view} = 1;
    } elsif (/^-keep$/) {
      $args{keep} = 1;
    } elsif (/^-wcet$/) {
      $ATQ_COMPUTE_MINMAX=2;
    } elsif (/^-debug$/ || /-verbose/) {
      $args{verbose} +=1;
	  $ENV{'ATTOLSTUDIO_VERBOSE'}=1;
    } elsif (/\+filter$/){
       $filter=1;
    } elsif (/^-/) {
      print "\nERROR: unknown option \"$_\"\n";
      help();
      exit 1;
    } elsif (/\.xrd$/i) {
      push (@xrd, $_); $reportFound=1;
    } elsif (/\.tzf$/i) {
      push (@tzf, $_);
    } elsif (/\.tgf$/i) {
      push (@tgf, $_); 
    } elsif (/\.ccf$/i) {
      push (@ccf, $_); 
    } elsif (/\.dcp$/i) {
      push (@dcp, $_); 
    } elsif (/\.tzf.html$/i) {
      push (@tzx, $_); $reportFound=1;
    } elsif (/\.tgf.html$/i) {
      push (@tgx, $_); $reportFound=1;
    } elsif (/\.dcp.html$/i) {
      push (@dcx, $_); $reportFound=1;
    } elsif (/\.tqf.html$/i) {
      push (@tqx, $_); $reportFound=1;
    } elsif (/\.rtx$/i) {
      push (@rtx, $_); $reportFound=1;
    } elsif (/\.crc$/i) {
      push (@crc, $_); $reportFound=1;
    } elsif (/\.met$/i) {
      push (@met, $_); $reportFound=1;
    } elsif (/\.fdc$/i) {
      push (@FDCFILES, $_); $reportFound=1;
    } elsif (/\.tio$/i) {
      push (@tio, $_);
    } elsif (/\.tsf$/i) {
      push (@TSFFILES, $_); $reportFound=1;
    } elsif (/\.tpf$/i) {
      push (@tpf, $_);
    } elsif (/\.tqf$/i) {
      push (@tqf, $_);
    } elsif (/\.tdf$/i) {
      push (@tdf, $_);
    } elsif (/\.bak$/i) {
    } elsif (/\.spt$/i) {
      my $f=$_;
      my @l=grep(/^\Q$f\E$/, @spt);
      push (@spt, $f) unless(scalar(@l));
      unless((-f $f) || ($theSptFile)){
	$theSptFile="\"$f\"" if(($instOpt=~ m/f/) || ($nbArgs));
	print "$f taken as on the fly report ($instOpt)\n" if(($args{verbose}) && ($theSptFile));
	$f=~s/\.spt$//;
	$args{NAME}=$f unless($f=~/\s/);
      }
    } elsif (($RESULT_EXT ne '') && (/\.$RESULT_EXT$/i)) {
      my $f=$_;
      if(-f $f) {
	my @l=grep(/^\Q$f\E$/, @spt);
	push (@spt, $f) unless(scalar(@l));
      }
    } elsif ((/\.log$/i) || (/\.xtp$/i)) {
      my $f=$_;
      reports ($f);
    } else {
       push(@unknownFiles,$_.' ');
    }
    $nbArgs-- if($nbArgs); # this allows to separate args from command line and from files
    unless(scalar(@ARGV) || $reportFound) {
      	print " then looking for sumary file\n";

  	$reportFound=1;
	if(-f "attolccfiles.log")
	{
		reports("attolccfiles.log");
	} elsif(-f "TestRTccfiles.log") {
		reports("TestRTccfiles.log");
	} elsif(-f "TestRTccfiles.xtp") {
		push(@ARGV,"TestRTccfiles.xtp");
	} elsif(-f "attolccReport.xtp") {
		push(@ARGV,"attolccReport.xtp");
	} else {
		print "WARNING: unknown file types: ",@unknownFiles if(@unknownFiles);
    		print "\nERROR: No report to show\n";
    		help();
    		exit 1;
	}
    }
  }

  $filter=0 if($clean);
  @xrd=sort @xrd;
  @crc=sort @crc;
  @met=sort @met;
  @FDCFILES=sort @FDCFILES;
  @TSFFILES=sort @TSFFILES;

  if($filter){
    if (($ENV{'ATLTGT'}) &&  (-r "$ENV{'ATLTGT'}/cmd/decode.pl")){
    	require "$ENV{'ATLTGT'}/cmd/decode.pl";
    }else{
          die &PerlTranslate("studioreport.pl", "decode option require decode.pl in the TDP. Not found\n","");
    }
  }
  foreach my $spt (@spt) {
	# Let's do it
	$spt=decode($spt) if($filter);
	my ($d,$n,$e) =SplitFileName($spt);
	$ATC_TIO_NAME=$d."attolcov.tio";   $ATC_TIO_NAME=$tio[0] if((@tio) && !( -f $ATC_TIO_NAME ));
	$ATP_TPF_NAME=$d."purifylt.tpf";   $ATP_TPF_NAME=$tpf[0] if((@tpf) && !( -f $ATP_TPF_NAME ));
	$ATQ_TQF_NAME=$d."quantifylt.tqf"; $ATQ_TQF_NAME=$tqf[0] if((@tqf) && !( -f $ATQ_TQF_NAME ));
	$ATG_TGF_NAME=$d."ccoupling.tgf";  $ATG_TGF_NAME=$tgf[0] if((@tgf) && !( -f $ATG_TGF_NAME ));
	$ATZ_TZF_NAME=$d."stacksize.tzf";  $ATZ_TZF_NAME=$tzf[0] if((@tzf) && !( -f $ATZ_TZF_NAME ));
	$TDFFILE     =$d."atltrace.tdf";   $TDFFILE=     $tdf[0] if((@tdf) && !( -f $TDFFILE ));
	if ( -f $ATC_TIO_NAME ) { $existed[$tio] = 1; }
	if ( -f $ATP_TPF_NAME ) { $existed[$tpf] = 1; }
	if ( -f $ATQ_TQF_NAME ) { $existed[$tqf] = 1; }
	if ( -f $ATG_TGF_NAME ) { $existed[$tgf] = 1; }
	if ( -f $ATZ_TZF_NAME ) { $existed[$tzf] = 1; }
	if ( -f $DCP_DCP_NAME ) { $existed[$dcp] = 1; }
	if ( -f $CCF_CCF_NAME ) { $existed[$ccf] = 1; }
	if ( -f $TDFFILE      ) { $existed[$tdf] = 1; }
    unless($clean) {
  	if(-f $spt) {
	  print "splitting $spt to $ATC_TIO_NAME $ATP_TPF_NAME $ATQ_TQF_NAME $TDFFILE $ATG_TGF_NAME $ATZ_TZF_NAME $DCP_DCP_NAME $CCF_CCF_NAME\n" if ($args{verbose});
	  if (SplitTraces($spt) != 0) {
	    exit(1);
	  }
	 $tdf_file_mode = "a" ; # for the next spt
	} else {
		if("\"$spt\"" eq $theSptFile){
			$existed[$tio]=1;
			$existed[$tpf]=1;
			$existed[$tqf]=1;
			$existed[$tgf]=1;
			$existed[$tzf]=1;
			$existed[$tdf]=1;
		}
		print &PerlTranslate("studioreport.pl", "%1 Not found. May be already split!\n","%1 is the file to split",$spt);
	}
    }

  if ( ! $used[$tio] && ! $existed[$tio] ) { unlink ($output_files[$tio]) if(defined $output_files[$tio]); } else {push(@tio,$ATC_TIO_NAME) if((@FDCFILES) && !(@tio)); }
  if ( ! $used[$tpf] && ! $existed[$tpf] ) { unlink ($output_files[$tpf]) if(defined $output_files[$tpf]); } else {push(@tpf,$ATP_TPF_NAME) if((@TSFFILES) && !(@tpf)); }
  if ( ! $used[$tqf] && ! $existed[$tqf] ) { unlink ($output_files[$tqf]) if(defined $output_files[$tqf]); } else {push(@tqf,$ATQ_TQF_NAME) if((@TSFFILES) && !(@tqf)); }
  if ( ! $used[$tgf] && ! $existed[$tgf] ) { unlink ($output_files[$tgf]) if(defined $output_files[$tgf]); } else {push(@tgf,$ATG_TGF_NAME) if((@TSFFILES) && !(@tgf)); }
  if ( ! $used[$tzf] && ! $existed[$tzf] ) { unlink ($output_files[$tzf]) if(defined $output_files[$tzf]); } else {push(@tzf,$ATZ_TZF_NAME) if((@TSFFILES) && !(@tzf)); }
  if ( ! $used[$dcp] && ! $existed[$dcp] ) { unlink ($output_files[$dcp]) if(defined $output_files[$dcp]); } else {push(@dcp,$DCP_DCP_NAME) if((@FDCFILES) && !(@dcp)); }
  if ( ! $used[$ccf] && ! $existed[$ccf] ) { unlink ($output_files[$ccf]) if(defined $output_files[$ccf]); } else {push(@ccf,$CCF_CCF_NAME) if((@FDCFILES) && !(@ccf)); }
  if ( ! $used[$tdf] && ! $existed[$tdf] ) { unlink ($output_files[$tdf]) if(defined $output_files[$tdf]); } else {push(@tdf,$TDFFILE)      if((@TSFFILES) && !(@tdf)); }
  }
  if($used[$tdf]) {
	$ATT_SPLIT=1;$ATT_LOOP = 1;$ATTOLHOME=$TESTRTDIR;$INTDIR=".";$BINDIR=GetBinDir();$BINDIR=ShortPathName($BINDIR) if($BINDIR=~/\s/);  $BINPATH=$BINDIR.$PATH_SEPARATOR;$BINPATH='' if($BINDIR=~/\s/);

	$ATT_SPLIT_PREFIX='att';
	$PATH_SEPARATOR = ($^O=~/win32/i) ? '\\' : '/';
	PostAttManage ($TDFFILE);
  }

  unlink ("$LOGFILENAME");

  if($clean) {
  	unlink(@rtx,@xrd,@tio,@tpf,@tqf,@tdf,@tgf,@tzf,@ccf,@dcp,@tgx,@dcx,@tzx,@spt,@tqx);
  	unlink(@crc,@met,@FDCFILES,@TSFFILES) if($clean >1);
  	print "clean done.\n";
  	exit(0);
  }

  $args{CRC} = join(',', @crc);
  $args{MET} = join(',', @met);
  $args{FDC} = join(',', @FDCFILES);
  $args{TSF} = join(',', @TSFFILES);
  $args{RTX} = join(',', @rtx);
  $args{XRD} = join(',', @xrd);
  $args{TIO} = join(',', @tio);
  $args{TPF} = join(',', @tpf);
  $args{TQF} = join(',', @tqf);
  $args{TDF} = join(',', @tdf);
  $args{TGF} = join(',', @tgf);
  $args{TZF} = join(',', @tzf);
  $args{CCF} = join(',', @ccf);
  $args{DCP} = join(',', @dcp);
  
  #place to post process if needed
  if (@tgf){
    $ATG_MODULE_AS=1;# tgf file is generated only for $ATG_MODULE_AS="Compilation Unit"(1)
    my $reportName="Ccoupling.tgf";
    $reportName=$args{NAME}.'.tgf' if(defined $args{NAME});
	my $reportfile=couplingPost($reportName,$args{TGF});
	push(@tgx,$reportfile);
  }
  if($ATGMODE && ($ATG_MODULE_AS==0) && (@tio) && (@ccf)){
    my $reportName="Ccoupling.tgf";
	$ATC_TIO_NAME=$tio[0];
    $reportName=$args{NAME}.'.tgf' if(defined $args{NAME});
	my $reportfile=couplingPost($reportName,$args{TIO});
	push(@tgx,$reportfile);    
  }
  if($DCPMODE && (@tio) && (@dcp)){
    my $reportName="Dcoupling.dcp";
	$ATC_TIO_NAME=$tio[0];
    $reportName=$args{NAME}.'.dcp' if(defined $args{NAME});
	my $reportfile=dataCouplingPost($reportName,$args{TIO});
	push(@dcx,$reportfile);
  }
  if (@tzf){
    my $reportName="stacksize.tzf";
    $reportName=$args{NAME}.'.tzf' if(defined $args{NAME});
	my $reportfile=stackSizePost($reportName,$args{TZF});
	push(@tzx,$reportfile);
  }
  if( $ATQMODE && ($ATQ_COMPUTE_MINMAX==2) && (@tqf)){
    my $reportName="wcet.tqf";
    $reportName=$args{NAME}.'.tqf' if(defined $args{NAME});
	my $reportfile=WCETPost($reportName,$args{TQF});
	push(@tqx,$reportfile);
  }

  $args{TGX} = join(',', @tgx);
  $args{DCX} = join(',', @dcx);
  $args{TZX} = join(',', @tzx);
  $args{TQX} = join(',', @tqx);
  $args{STUDIO} = join(' ',@studio_args).'  '.$theSptFile;#' "'.join('" "', @xrd, @crc, @met, @FDCFILES, @tio, @TSFFILES, @tpf, @tqf, @tdf, @rtx ).'"';
  $run="-execute" if($lmth);

  return 0; #%args;
}


##############################################################################
# Generate project

sub generateProject {
  my $typeCode="487543129";
  $nodeNumber=239471645;
  my $targetFullPath=$ENV{ATLTGT};
  my $d='cgnu';
  my $xdpName;
  unless($targetFullPath){
    if(opendir(DIR1,"$ENV{TESTRTDIR}/targets")){
      while($d=readdir(DIR1)) {
    	next if ($d=~/^\./);
    	next if ($d=~/^\.\./);
    	next unless ($d=~/^c/);
    	last;
      }
      closedir(DIR1);
    }
    $targetFullPath=$ENV{TESTRTDIR}.'/targets/'.$d;
  }
  ($d,$xdpName) = SplitFileName($targetFullPath);

  open (PRJ, ">$dirtp/${projectName}.rtp") || die "Cannot open the project \"$dirtp/${projectName}.rtp\"\n";
  print PRJ <<"END";
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE project SYSTEM "TestRealTime/project.dtd" >
<project>
  <group>
    <node>
      <data>
        <name>${projectName}</name>
        <quid>239471478</quid>
        <renameable>false</renameable>
        <deletable>false</deletable>
        <open>true</open>
        <sorted>false</sorted>
      </data>
      <current_configuration>viewer</current_configuration>
      <state>0</state>
      <background>false</background>
      <configuration_list>
        <configuration>
          <name>viewer</name>
          <executionFlag>24127</executionFlag>
        </configuration>
      </configuration_list>
    </node>
    <group_child>
END

  print PRJ <<"END";
      <application>
       <test>
        <node>
          <data>
            <name>$args{NAME}</name>
            <quid>$nodeNumber</quid>
            <renameable>true</renameable>
            <deletable>true</deletable>
            <open>true</open>
            <sorted>false</sorted>
          </data>
          <current_configuration>viewer</current_configuration>
          <state>0</state>
          <background>false</background>
          <outfilename_list>
END
  foreach my $type (qw(XRD CRC FDC TIO TSF TPF TQF TDF DCP CCF TGX DCX TZX MET RTX)) {
   if($args{$type} ne '') {
   $args{$type}=~s!,\Q$dirtp\E[\\\/]!,.\/!g;
   $args{$type}=~s!^\Q$dirtp\E[\\\/]!.\/!;
    print PRJ <<"END";
              <outfilename>
                <filetype>${typeCode}_${type}</filetype>
                <filepath>$args{$type}</filepath>
              </outfilename>
END
   }
}
  print PRJ <<"END";
          </outfilename_list>
          <configuration_list>
            <configuration>
              <name>viewer</name>
            </configuration>
          </configuration_list>
        </node>
        <group_child>
        </group_child>
       </test>
      </application>
END
$nodeNumber++;



  print PRJ <<"END";
    </group_child>
  </group>
  <object_file>${projectName}.inf</object_file>
  <sort_method>0</sort_method>
  <root_configuration_list>
    <configuration>
      <name>viewer</name>
      <envVariables>$envVariables</envVariables>
      <intdir>.</intdir>
      <outdir>.</outdir>
      <targetPath>$xdpName</targetPath>
      <targetPathAsRelative>false</targetPathAsRelative>
      <targetFullPath>$targetFullPath</targetFullPath>
      <iniFileName>tp.ini</iniFileName>
      <executionFlag>24127</executionFlag>
    </configuration>
  </root_configuration_list>
  <version_file>1</version_file>
</project>
END

  close (PRJ);

  # Generate .inf file
  open (INF, ">$dirtp/${projectName}.inf") || die "Cannot open the project \"$dirtp/${projectName}.inf\"\n";
  print INF <<"END";
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE objects SYSTEM "C:/Program files/Rational/TestRealTime/lib/project.dtd" >
<objects>
  <version_file>1</version_file>
</objects>

END
  close (INF);

}

##############################################################################
# Run studio with the project.

sub runStudio {
  my $command = "studio ". $args{STUDIO};
    $command .= " $run $args{NAME} \"$dirtp/${projectName}.rtp\" ";
  print "Running the command \n$command\n" if ($args{verbose});
  system ($command);
}

##############################################################################
# remove temporary project
sub removeProject {
  unless ($args{keep}) {
    unlink ("$dirtp/${projectName}.rtp");
    unlink ("$dirtp/${projectName}.inf");
  }
}

##############################################################################
# Main program
sub main {
  parseArgs();
  generateProject();
  runStudio();
  #sleep(10);
  removeProject();
}

###########################################################################

main();
1;
